using LinearAlgebra
using Statistics
using LaTeXStrings
using Random
using Flux
import PyPlot as plt

include("readclassjson.jl")
include("rerm_lin_reg.jl")

data = readclassjson("residual_props.json")
X_train = data["X_train"]
X_test = data["X_test"]
y_train = data["y_train"]
y_test = data["y_test"]

X_train = [ones(500) X_train]
X_test = [ones(500) X_test]

#############################
# You only need to edit code in this section

# Root mean square error function
function RMSE(yhat, y)
    #TODO: IMPLEMENT THIS FUNCION
end

# Tilted absolute penalty function
function p_tlt(r, tau)
    #TODO: IMPLEMENT THIS FUNCION
end

# Tilted loss function with tau = 0.15
function loss_tlt_15(yhat, y)
    #TODO: IMPLEMENT THIS FUNCION
end

# Tilted loss function with tau = 0.5
function loss_tlt_50(yhat, y)
    #TODO: IMPLEMENT THIS FUNCION
end

# Tilted loss function with tau = 0.85
function loss_tlt_85(yhat, y)
    #TODO: IMPLEMENT THIS FUNCION
end

none_reg(x) = 0

#############################

losses = [loss_tlt_15, loss_tlt_50, loss_tlt_85]
names = ["tlt_15", "tlt_50", "tlt_85"]

train_rmses = Dict()
test_rmses = Dict()
train_res = Dict()
test_res = Dict()

for (LOSS, NAME) in zip(losses, names)
    theta = rerm_lin_reg(X_train, y_train, LOSS, none_reg, 1)

    train_rmses[NAME] = RMSE(X_train*theta, y_train)
    test_rmses[NAME] = RMSE(X_test*theta, y_test)
    train_res[NAME] = X_train*theta - y_train
    test_res[NAME] = X_test*theta - y_test
end

display(train_rmses)
display(test_rmses)

t_15_train = mean(train_res["tlt_15"] .> 0)
t_15_test = mean(test_res["tlt_15"] .> 0)
t_50_train = mean(train_res["tlt_50"] .> 0)
t_50_test = mean(test_res["tlt_50"] .> 0)
t_85_train = mean(train_res["tlt_85"] .> 0)
t_85_test = mean(test_res["tlt_85"] .> 0)

println("% Overestimates for tau = 0.15:")
println("\t On training data = ", t_15_train)
println("\t On testing data = ", t_15_test)

println("% Overestimates for tau = 0.50:")
println("\t On training data = ", t_50_train)
println("\t On testing data = ", t_50_test)

println("% Overestimates for tau = 0.85:")
println("\t On training data = ", t_85_train)
println("\t On testing data = ", t_85_test)

d = length(train_res["tlt_15"])
y = collect(1:d)/d
    
fig, ax = plt.subplots(2,1)
taus = [0.15, 0.50, 0.85]

ax[1].set_title("Train")
for (i,resname) in enumerate(["tlt_15", "tlt_50", "tlt_85"])
    res = train_res[resname]
    ax[1].plot(sort(res), y, label=string(L"\tau = ", taus[i]))    
end
ax[1].legend()

ax[2].set_title("Test")
for (i,resname) in enumerate(["tlt_15", "tlt_50", "tlt_85"])
    res = test_res[resname]
    ax[2].plot(sort(res), y, label=string(L"\tau = ", taus[i]))    
end
ax[2].legend()
plt.savefig("residual_cdfs.pdf", bbox_inches="tight")
